/******************************************************************
Gesrtk.c

	Demonstrates the use of the gesuprt with the 
	Simplified Real-Time Kernel (SRTK)
	and costatements. The SRTK is a multi-tasking operating system
   with 3 levels of task priority.

	The high priority task is the function srtk_hightask(). This
	function is defined by the user, but called from the SRTK every
	25 ms. The low priority task, srtk_lowtask(), is called every
	100 ms and can be preempted by the high priority task. 	  

	A background task could be run that could be preempted by
	either SRTK task.
**********************************************************************/

/***** The next four macros determine what serial port to use, Though you can have multiple ports
		selected, for the purpose of this sample, Only one port is set up.

******/
#define	GE_USE_PORTZ1
//#define	GE_USE_PORTZ0
//#define	GE_USE_PORTA
//#define	GE_USE_PORTB

/**** Choose the baudrate here ****/
#define	BAUDRATE		9600

#use		GESUPRT.LIB						// support library for G.E.
#use		vdriver.lib						// virtual library

/**** These #ifdef statements choose the port from the above four selections ****/
#ifdef	GE_USE_PORTZ1
#define	PORT	GE_PORTZ1
#endif

#ifdef	GE_USE_PORTZ0
#define	PORT	GE_PORTZ0
#endif

#ifdef	GE_USE_PORTA
#define	PORT	GE_PORTA
#endif

#ifdef	GE_USE_PORTB
#define	PORT	GE_PORTB
#endif

#use srtk.lib								//  to
#define RUNKERNEL 1						//  run SRTK

int HCOUNT, LCOUNT;
char DispStr[50];
main(){
		
		HCOUNT = LCOUNT = 0;
		VdInit();							// virtual driver needed by SRTK
		init_srtkernel();  				// initialize the SRTK
		GEInit(PORT,BAUDRATE);			// initialize the gesuprt
		GEStopMacro(PORT);				// Stop Any Playing Macros
		GEBackLight(PORT,1);				// Turn on the Backlight
		GESetContrast(PORT,35);			// Set the Contrast
		GEClearScreen(PORT);				// Clear the Screen
		GESetFont(PORT,2);				// Set the Font that is used with GEPutText
		GEPutRectangle(PORT,0,40,30,280,210);
		GEPutText(PORT,"SRTK SAMPLE",100,90);	// display Header 1
		GEPutText(PORT,"-----------",100,110);// Display dashes
		GEPutText(PORT,"HCOUNT LCOUNT",90,130);// Display header 2
		
		// loop while SRTK does the work
		while(1);
		
}


//-------------- The Real Time Kernel Tasks ---------------- 


// This high priority task is executed every 25 ms

#funcchain _srtk_hightask srtk_hightask1

srtk_hightask1()
{

	// Increment counter on each pass
	HCOUNT++;
}


// This low priority task is executed every 100 ms

#funcchain _srtk_lowtask srtk_lowtask1

srtk_lowtask1()
{

	// Increment counter on each pass
	LCOUNT++;
	costate
	{
		waitfor(DelayMs(500));
		sprintf(DispStr,"%3d    %3d", HCOUNT, LCOUNT);
		waitfor(	GEPutTextWF(PORT,DispStr,100,150)	);
	}
	costate
	{
		// When HCOUNT nears 1000, reset counters
		waitfor(  HCOUNT >= 960 );
		HCOUNT = 0;
		LCOUNT = 0;
	}
}


