/*** GE_Macro.c ***/

/*** This sample demonstrates macros, buttons
	  The virtual keyboard, and unsolicited packet
	  response retrieving in both a sequential and
	  cooperative multitasking environment.
	  
	  This sample uses the following G.E. functions (either WF or normal)
	  GEInit
	  GESuperResetWF
	  GEClearScreenWF
	  GEDefineButtonWF
	  GEBeginMacroWF
	  GEEndMacroWF
	  GESetFloatWF
	  GESetLongWF
	  GESetStringWF
	  GESendFloatWF
	  GESendLongWF
	  GESendStringWF
	  GELockBufferWF
	  GEGetPcktWF
	  GEKeyboardMacroWF
	  GEDisplayButtonWF
	  GEPlayMacroWF
	  GEStopMacroWF
	  GELinkBtnToMacWF
	  GESetContrastWF
	  GEBackLightWF
	  
	  
***/


#define	USE_WAITFOR		// uncomment for using the cooperative sample
								// comment for using the sequential sample




/***** Though more than 1 port can be used, for purposes of this sample
		 please choose one of the four following macros to for communications
		 with the G.E. device.
****/								
								
#define	GE_USE_PORTZ1
//#define	GE_USE_PORTZ0
//#define	GE_USE_PORTA
//#define	GE_USE_PORTB

/**** Choose the baudrate here ****/
#define	BAUDRATE		9600

#use		GESUPRT.LIB						// support library for G.E.
#use		vdriver.lib						// virtual library

/**** These #ifdef statements choose the port from the above four selections ****/
#ifdef	GE_USE_PORTZ1
#define	PORT	GE_PORTZ1
#endif

#ifdef	GE_USE_PORTZ0
#define	PORT	GE_PORTZ0
#endif

#ifdef	GE_USE_PORTA
#define	PORT	GE_PORTA
#endif

#ifdef	GE_USE_PORTB
#define	PORT	GE_PORTB
#endif
/*******************************************************************************/


/***** Some misc. definitions   ********/
#define	BTN_FLOAT		1
#define	BTN_LONG			2
#define	BTN_STRING		3
#define	MAC_FLOAT		1
#define	MAC_LONG			2
#define	MAC_STRING		3
#define	MAC_MAIN			4
#define	CMD_VKENTER		14
#define	CMD_BTN			13
#define	VK_LONGFLAG			0xb			
#define	VK_FLOATFLAG		0x13					
#define  VK_STRINGFLAG		0x3	




char check;
char	Packet[255];
char	sVal[25];
float	fVal;
long	lVal;
long count;


/******** The waitfor version ********************************/
#ifdef	USE_WAITFOR
CoData	_DefMacs,_DefBtns,_DefLinks,_MainCostate;
main ()
{

	count = 0;
	VdInit();
	GEInit(PORT,BAUDRATE);
	CoBegin (&_DefBtns);
	for (;;)
	{
		costate	_DefBtns
		{
			
			waitfor( GEStopMacroWF(PORT) );				// Stop any playing macro
			waitfor( GESuperResetWF(PORT) );				// Glear out all bitmaps, buttons, and fonts
			waitfor( GEBackLightWF(PORT,1) );			// Turn on the Backlight
			waitfor( GESetContrastWF(PORT,40) );		// Set the contrast		
			/***** GEDefineButtonWF in detail *****/
			waitfor( GEDefineButtonWF(PORT,				// The port to use 
									   BTN_FLOAT,				// The button Id number
											  0x5F,				// the button properties (see function desc. for details)
											  	  1,				// Top left cell location
											  	 10,				// Bottom right cell location
											  	  0,				// the Bitmap ID number if used
											  	   "ENTER\nFLOAT\nNUMBER" // the button text if used
											  	   ));
											  	   
			waitfor( GEDefineButtonWF(PORT, BTN_LONG, 0x5F, 4, 13, 0, "ENTER\nLONG\nNUMBER") );
			waitfor( GEDefineButtonWF(PORT, BTN_STRING, 0x5F, 7, 16, 0, "ENTER\nSTRING\nVAR.") );
			
			
			CoBegin (&_DefMacs);
		}
		
		costate _DefMacs
		{
			waitfor ( GEBeginMacroWF(PORT,MAC_FLOAT) );					// Start of macro command
			waitfor ( GESetFloatWF(PORT,1.25000) );			// Set the Data entry window of the V.Kboard
			
			/****** Keyboard Macro description in detail ************/
			waitfor ( GEKeyBoardMacroWF(PORT,					// The Port to use 
									 "ENTER FLOAT",					// The Top description
									 				 1,					// Enable (1) or Disable (0)
									  VK_FLOATFLAG,					// Keyboard properties (see function desc. for details)
									          NULL,					// Max user input if used
									          NULL						// Min user input if used
									          ) );
									          
			waitfor ( GEEndMacroWF(PORT,MAC_FLOAT) );					// End of macro command
			
			waitfor ( GEBeginMacroWF(PORT,MAC_LONG) );					// Start of macro command
			waitfor ( GESetLongWF(PORT,1234567) );				// Set the data entry window of the V.Kboard
			waitfor ( GEKeyBoardMacroWF(PORT, "ENTER LONG",1,VK_LONGFLAG,NULL,NULL) );
			waitfor ( GEEndMacroWF(PORT,MAC_LONG) );					// End of macro command
			
			waitfor ( GEBeginMacroWF(PORT,MAC_STRING) );					// Start of macro command
			waitfor ( GESetStringWF(PORT,"STRING") );			// Set the data entry window of the V.Kboard
			waitfor ( GEKeyBoardMacroWF(PORT, "ENTER STRING",1,VK_STRINGFLAG,NULL,NULL) );
			waitfor ( GEEndMacroWF(PORT,MAC_STRING) );					// End of macro command
			
			waitfor ( GEBeginMacroWF(PORT,MAC_MAIN) );					// Start of macro command		
			waitfor ( GELockBufferWF(PORT,0) );					// Lock the display buffer
			waitfor ( GEClearScreenWF(PORT) );					// Clear the display
			waitfor ( GEDisplayButtonWF(PORT,BTN_FLOAT) );				// Display Button 1
			waitfor ( GEDisplayButtonWF(PORT,BTN_LONG) );				// Display Button 2
			waitfor ( GEDisplayButtonWF(PORT,BTN_STRING) );				// Display Button 3	
			waitfor ( GELockBufferWF(PORT,1) );					// Unlock Display buffer
			waitfor ( GEEndMacroWF(PORT,MAC_MAIN) );					// End of macro command
			
			
			CoBegin (&_DefLinks);
			
		}
		costate	_DefLinks
		{
			waitfor ( GELinkBtnToMacWF(PORT,1,BTN_FLOAT,MAC_FLOAT) );					// Link button 1 press to run a Macro  1
			waitfor ( GELinkBtnToMacWF(PORT,1,BTN_LONG,MAC_LONG) );					// Link button 2 press to run a Macro  2
			waitfor ( GELinkBtnToMacWF(PORT,1,BTN_STRING,MAC_STRING) );					// Link button 3 press to run a Macro  3
			CoBegin (&_MainCostate);
		}
		costate  _MainCostate
		{
			waitfor (GEPlayMacroWF(PORT,MAC_MAIN) );						// Play Macro number 4
		}
		costate 
		{
			waitfor ( GEGetPcktWF(PORT,Packet) );					// get an unsolicited response if any
			if (Packet[CMD] == CMD_BTN)								// check to see if it is a button press
			{
				printf("ButtonPressed = %d\n",Packet[DATA0]);	// Display the Button pressed value in the stdio window
				check = Packet[DATA0];									// Set the check value to the Button Pressed number	
			}
						
			if (Packet[CMD] == CMD_VKENTER)							// check to see if it is the enter key of the V.K.
			{
				switch (check)		
				{
					case BTN_FLOAT:
						waitfor(GESendFloatWF(PORT,&fVal) );		// Get the float value entered from the V.K.
						printf("Float Entered = %f\n",fVal);		// display it in stdio window
					break;
					case BTN_LONG:
						waitfor(GESendLongWF(PORT,&lVal) );			// Get the long value entered from the V.K.
						printf("Long Entered = %ld\n",lVal);		// display it in stdio window
					break;
					case BTN_STRING:
						waitfor(GESendStringWF(PORT,sVal) );		// Get the string value entered from the V.K.
						printf("String Entered = %s\n",sVal);		// display it in the stdio window
					break;
				}
			}
		}
		
		/**** Routine for simply incrementing a counter, this shows
		cooperative multitasking ability  ****/
		
		costate
		{
			waitfor(DelayMs(100));
			count++;
			printf("New Count = %ld\r",count);
		}
	}
}


/*********** The sequential version, Does the same thing as above, see the above function
				 for comments										**********************************/
#else

main ()
{
	int loading;
	long	Del;
	Del = MS_TIMER + 100;
	loading = 1;
	
	count = 0;
	VdInit();
	GEInit(PORT,BAUDRATE);
	for (;;)
	{
			
		if (loading == 1)
		{
			GEStopMacro(PORT);
			GESuperReset(PORT);
			GEBackLight(PORT,1);
			GESetContrast(PORT,40);
			GEDefineButton(PORT, 1, 0x5F, 1, 10, 0, "ENTER\nFLOAT\nNUMBER");
			GEDefineButton(PORT, 2, 0x5F, 4, 13, 0, "ENTER\nLONG\nNUMBER");
			GEDefineButton(PORT, 3, 0x5F, 7, 16, 0, "ENTER\nSTRING\nVAR.");
			GEBeginMacro(PORT,1);
			GESetFloat(PORT,1.25000);
			GEKeyBoardMacro(PORT, "ENTER FLOAT",1,VK_FLOATFLAG,NULL,NULL);
			GEEndMacro(PORT,1);
				
			GEBeginMacro(PORT,2);
			GESetLong(PORT,1234567);
			GEKeyBoardMacro(PORT, "ENTER LONG",1,VK_LONGFLAG,NULL,NULL);
			GEEndMacro(PORT,2);
				
			GEBeginMacro(PORT,3);
			GESetString(PORT,"STRING");
			GESetChar(PORT,' ');
			GESetChar(PORT,'A');
			GEKeyBoardMacro(PORT, "ENTER STRING",1,VK_STRINGFLAG,NULL,NULL);
			GEEndMacro(PORT,3);
			
			GEBeginMacro(PORT,4);
			GELockBuffer(PORT,0);
			GEClearScreen(PORT);
			GEDisplayButton(PORT,1);
			GEDisplayButton(PORT,2);
			GEDisplayButton(PORT,3);
			GELockBuffer(PORT,1);
			GEEndMacro(PORT,4);
			
			GELinkBtnToMac(PORT,1,1,1);
			GELinkBtnToMac(PORT,1,2,2);
			GELinkBtnToMac(PORT,1,3,3);
			loading = 0;
			GEPlayMacro (PORT,4);
		}	
		
		else
		{	
			if ( GEGetPckt(PORT,Packet) )
			{
				if (Packet[CMD] == CMD_BTN)
					{
					printf("ButtonPressed = %d\n",Packet[DATA0]);
					check = Packet[DATA0];
				}
				if (Packet[CMD] == CMD_VKENTER)
				{
					printf("Enter Key Pressed\n");
					switch (check)
					{
						case BTN_FLOAT:
							GESendFloat(PORT,&fVal);
							printf("Float Entered = %f\n",fVal);
						break;
						case BTN_LONG:
							GESendLong(PORT,&lVal);
							printf("Long Entered = %ld\n",lVal);
						break;
						case BTN_STRING:
							GESendString(PORT,sVal);
							printf("String Entered = %s\n",sVal);
						break;
					}
				}
			}

		}
			printf ("count = %ld\r",count);
			count++;

		
	}
}
#endif
/*************************************************************************/